#include <gtk/gtk.h>
#include <stdlib.h>
#include "ruleadd.h"


#define TABLE_ROWS 7
#define TABLE_COLS 7
#define MAX_BURST 65535
#define MAX_DELAY 65535

#define MTU 1500

struct entries{
	GtkWidget *type;
	GtkWidget *proto;
	GtkWidget *min_size;
	GtkWidget *max_size;
	GtkWidget *probability;
	GtkWidget *max_burst;
	GtkWidget *burst_skew;
	GtkWidget *max_delay;
	GtkWidget *delay_skew;
	/* need to carry over data from the supplied rule */
	int operation;
	int rule_num;
	/* reference to this window, so we can close it */
	GtkWidget *window;
};


/* forms a rule from the entries */
static void form_rule(struct rule_params *r, struct entries *e){
	GtkWidget *tmp;
	char *label;

	r->operation = e->operation;
	r->rule_num = e->rule_num;

	/* work out the rule type, from the gtk_option_menu */
	tmp = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(
							  GTK_OPTION_MENU(e->type))));
	label = (char *)gtk_widget_get_name(tmp);
	if (strcmp(label, "DROP") == 0) r->rule_type = DROP;
	else if (strcmp(label, "REORDER") == 0) r->rule_type = REORDER;
	else return;

	/* work out the protocol from the gtk_option_menu */
	tmp = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(
							  GTK_OPTION_MENU(e->proto))));
	label = (char *)gtk_widget_get_name(tmp);
	if (strcmp(label, "ALL") == 0) r->proto = PROTO_ALL;
	else if (strcmp(label, "TCP") == 0) r->proto = PROTO_TCP;
	else if (strcmp(label, "UDP") == 0) r->proto = PROTO_UDP;
	else if (strcmp(label, "ICMP") == 0) r->proto = PROTO_ICMP;
	else if (strcmp(label, "IGMP") == 0) r->proto = PROTO_IGMP;
	else return;

	/* read in spin button settings */
	r->min_size = (int)gtk_spin_button_get_value_as_int(
											   GTK_SPIN_BUTTON(e->min_size));
	r->max_size = (int)gtk_spin_button_get_value_as_int(
											   GTK_SPIN_BUTTON(e->max_size));
	/* prob * 10, to get chance in a thousand */
	r->probability = 10 * gtk_spin_button_get_value_as_float(
											   GTK_SPIN_BUTTON(e->probability));

	if (r->rule_type == REORDER){
		r->max_burst_size = (int)gtk_spin_button_get_value_as_int(
												GTK_SPIN_BUTTON(e->max_burst));
		r->burst_skew = (int)gtk_spin_button_get_value_as_int(
												GTK_SPIN_BUTTON(e->burst_skew));
		r->max_delay_size = (int)gtk_spin_button_get_value_as_int(
												GTK_SPIN_BUTTON(e->max_delay));
		r->delay_skew = (int)gtk_spin_button_get_value_as_int(
												GTK_SPIN_BUTTON(e->delay_skew));
	}
	else{
		r->max_burst_size = r->burst_skew = 
			r->max_delay_size = r->delay_skew = 0;
	}
}

static void ok_pressed(GtkWidget *widget, gpointer data){
	struct rule_params *r = malloc(sizeof(struct rule_params));
	form_rule(r, (struct entries *)data);

	if (valid_rule(r)){
		rule_in(r);
		gtk_widget_destroy( ((struct entries *)data)->window);
	}
	else{
		display_message("Values Invalid");
/*
		g_print("rule_type: %d\n", r->rule_type);
		g_print("operation: %d\n", r->operation);
		g_print("proto: %d\n", r->proto);
		g_print("rule_num: %d\n", r->rule_num);
		g_print("probability: %d\n", r->probability);
		g_print("min_size: %d\n", r->min_size);
		g_print("max_size: %d\n", r->max_size);
		g_print("max_delay_size: %d\n", r->max_delay_size);
		g_print("delay_skew: %d\n", r->delay_skew);
		g_print("max_burst_size: %d\n", r->max_burst_size);
		g_print("burst_skew: %d\n", r->burst_skew);
*/
	}
	free(r);

	/* destroy this window */
}

/* called when cancel hit, or window closed */
/* data is the window */
static void cancel_pressed(GtkWidget *widget, gpointer data){
	gtk_widget_destroy(GTK_WIDGET(data));
}


/* called to display a window to add/update/insert a rule */
void addrule_window(struct rule_params *rule){
	GtkWidget *window;
	GtkWidget *main_box;
	GtkWidget *table;
	GtkWidget *button_box;

	GtkWidget *label, *button, *tmp; /* temporary variables */

	/* data entries, passed as argument when OK button pressed  */
	struct entries *data = malloc(sizeof(struct entries));


	/* sanity check */
	if (rule == NULL){
		display_message("Error: Null encounted");
		return;
	}
	if (rule->operation == UPDATE && !valid_rule(rule)){ 
		/* an update should be valid already*/
		display_message("Error: Update invalid");
		return;
	}
	else if (!(rule->operation == APPEND || rule->operation == INSERT
			|| rule->operation == UPDATE)){
		display_message("Error: Operation not permitted");
		return;
	}

	/* preseve important data */
	data->operation = rule->operation;
	data->rule_num = rule->rule_num;

	/* if not updating, set rule elements appropriately */
	if (rule->operation != UPDATE){
		rule->rule_type = DROP;
		rule->proto = PROTO_ALL;
		rule->min_size = 0;
		rule->max_size = MTU;
		rule->max_burst_size = 1;
		rule->burst_skew = 0;
		rule->max_delay_size = 1;
		rule->delay_skew = 0;
		rule->probability = 500;
	}

	/* setup the main compoments */
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	data->window = window; /* store reference, for later destruction */
	main_box = gtk_vbox_new(0, 20);
	button_box = gtk_hbox_new(1, 20);

	table = gtk_table_new(TABLE_ROWS, TABLE_COLS, 0);

	gtk_container_add(GTK_CONTAINER(window), main_box);

	/* pack table into main_box */
	gtk_box_pack_start(GTK_BOX(main_box), table, 0, 0, 20);
	gtk_box_pack_start(GTK_BOX(main_box), button_box, 0, 0, 0);




	/**********************************
	* add labels and entries 
	************************************/
	gtk_table_attach_defaults(GTK_TABLE(table), 
							  gtk_label_new("Type:"), 0, 1, 0, 1);

	/* menu for rule type */
	data->type = gtk_option_menu_new();
		label = gtk_menu_new();
		tmp = gtk_menu_item_new_with_label("DROP");
		gtk_widget_set_name(tmp, "DROP");
		gtk_menu_append(GTK_MENU(label), tmp);
		tmp = gtk_menu_item_new_with_label("REORDER");
		gtk_widget_set_name(tmp, "REORDER");
		gtk_menu_append(GTK_MENU(label), tmp);
		gtk_option_menu_set_menu(GTK_OPTION_MENU(data->type), label);
	gtk_table_attach_defaults(GTK_TABLE(table), data->type, 1, 2, 0, 1);

	/* if reordering, set value according accordingly */
	if (rule->rule_type == REORDER){
		gtk_option_menu_set_history(GTK_OPTION_MENU(data->type), 1);
	}


	gtk_table_attach_defaults(GTK_TABLE(table), 
							  gtk_label_new("Protocol:"), 3, 4, 0, 1);

	data->proto = gtk_option_menu_new();
	label = gtk_menu_new();
	tmp = gtk_menu_item_new_with_label("ALL");
	gtk_widget_set_name(tmp, "ALL");
	gtk_menu_append(GTK_MENU(label), tmp);
	tmp = gtk_menu_item_new_with_label("TCP");
	gtk_widget_set_name(tmp, "TCP");
	gtk_menu_append(GTK_MENU(label), tmp);
	tmp = gtk_menu_item_new_with_label("UDP");
	gtk_widget_set_name(tmp, "UDP");
	gtk_menu_append(GTK_MENU(label), tmp);
	tmp = gtk_menu_item_new_with_label("ICMP");
	gtk_widget_set_name(tmp, "ICMP");
	gtk_menu_append(GTK_MENU(label), tmp);
	tmp = gtk_menu_item_new_with_label("IGMP");
	gtk_widget_set_name(tmp, "IGMP");
	gtk_menu_append(GTK_MENU(label), tmp);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(data->proto), label);
	gtk_table_attach_defaults(GTK_TABLE(table), data->proto, 4, 5, 0, 1);

	/* set correct value of the drop down list */
	if (rule->proto == PROTO_TCP) 
		gtk_option_menu_set_history(GTK_OPTION_MENU(data->proto), 1);
	if (rule->proto == PROTO_UDP) 
		gtk_option_menu_set_history(GTK_OPTION_MENU(data->proto), 2);
	if (rule->proto == PROTO_ICMP) 
		gtk_option_menu_set_history(GTK_OPTION_MENU(data->proto), 3);
	if (rule->proto == PROTO_IGMP) 
		gtk_option_menu_set_history(GTK_OPTION_MENU(data->proto), 4);


	/* size row */
		label = gtk_label_new("Min Size:");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
	data->min_size = gtk_spin_button_new(
			GTK_ADJUSTMENT(gtk_adjustment_new(
									rule->min_size, 1.0, MTU, 1.0, 10.0, 10.0)),
			1.0, 0);
	gtk_editable_set_editable(GTK_EDITABLE(data->min_size), 1);
	gtk_table_attach_defaults(GTK_TABLE(table), data->min_size, 1, 2 , 2, 3);
		label = gtk_label_new("bytes");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 2, 3);
		label = gtk_label_new("Max Size:");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 4, 5, 2, 3);
	data->max_size = gtk_spin_button_new(
			GTK_ADJUSTMENT(gtk_adjustment_new(
								   rule->max_size, 1.0, MTU, 1.0, 10.0, 10.0)),
			1.0, 0);
	gtk_editable_set_editable(GTK_EDITABLE(data->max_size), 1);
	gtk_table_attach_defaults(GTK_TABLE(table), data->max_size, 5, 6 , 2, 3);
		label = gtk_label_new("bytes");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 6, 7, 2, 3);

	/* probability row */
		label = gtk_label_new("Probability:");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3, 4);
	data->probability = gtk_spin_button_new(
		GTK_ADJUSTMENT(gtk_adjustment_new(
						rule->probability/10, 0.1, 100.0, 1.0, 10.0, 10.0)),
		1.0, 1);
	gtk_editable_set_editable(GTK_EDITABLE(data->probability), 1);
	gtk_table_attach_defaults(GTK_TABLE(table), data->probability, 1, 2, 3, 4);
		label = gtk_label_new("%");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 3, 4);

	/* burst row */
		label = gtk_label_new("Max Burst:");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 5, 6);
	data->max_burst = gtk_spin_button_new(
		GTK_ADJUSTMENT(gtk_adjustment_new(
						rule->max_burst_size, 1.0, MAX_BURST, 1.0,10.0, 10.0)),
		1.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), data->max_burst, 1, 2, 5, 6);
		label = gtk_label_new("bytes");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 5, 6);
		label = gtk_label_new("Skew");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 4, 5, 5, 6);
	data->burst_skew = gtk_spin_button_new(
		GTK_ADJUSTMENT(gtk_adjustment_new(
							 rule->burst_skew, -100.0, 100.0, 1.0, 10.0, 10.0)),
		1.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), data->burst_skew, 5, 6, 5, 6);
		label = gtk_label_new("%");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 6, 7, 5, 6);

	/* delay row */
		label = gtk_label_new("Max Delay:");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 6, 7);
	data->max_delay = gtk_spin_button_new(
		GTK_ADJUSTMENT(gtk_adjustment_new(
					   rule->max_delay_size, 1.0, MAX_BURST, 1.0,10.0, 10.0)),
		1.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), data->max_delay, 1, 2, 6, 7);
		label = gtk_label_new("bytes");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 6, 7);
		label = gtk_label_new("Skew");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 4, 5, 6, 7);
	data->delay_skew = gtk_spin_button_new(
		GTK_ADJUSTMENT(gtk_adjustment_new(
					   rule->delay_skew, -100.0, 100.0, 1.0, 10.0, 10.0)),
		1.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), data->delay_skew, 5, 6, 6, 7);
		label = gtk_label_new("%");
		gtk_table_attach_defaults(GTK_TABLE(table), label, 6, 7, 6, 7);

	/* set some sensible cell sizes */
	gtk_widget_set_usize(data->type, 120, 30);
	gtk_widget_set_usize(data->proto, 60, 30);
	gtk_widget_set_usize(data->max_size, 60, 30);

	

	/* create button box */
	button = gtk_button_new_with_label("OK");
	gtk_box_pack_start(GTK_BOX(button_box), button, 0, 0, 25);
	gtk_signal_connect(GTK_OBJECT(button), "clicked", 
					   GTK_SIGNAL_FUNC(ok_pressed), data);

	button = gtk_button_new_with_label("Cancel");
	gtk_box_pack_start(GTK_BOX(button_box), button, 0, 0, 25);
	gtk_signal_connect(GTK_OBJECT(button), "clicked", 
					   GTK_SIGNAL_FUNC(cancel_pressed), window);


	/* closing window should be like a cancel */
	gtk_signal_connect(GTK_OBJECT(window), "destroy", 
					   GTK_SIGNAL_FUNC(cancel_pressed), window);

	gtk_widget_show_all(window);

	/* free memory used by rules */
	free(rule);
}
