/*
 * Module that forms part of a controllable faulty network router
 *
 * Completed as part of a final year project in the 
 *	Department of Computer Science,
 *	University of Wales, Aberstwyth
 *
 * (C) 2003 Thomas Salmon, this code is GNU GPL
 */

#include <linux/skbuff.h>
#include <linux/in.h>


/* types of rule that may be used */
#define DROP	1
#define REORDER	2

/* operation types */
#define APPEND	0
#define INSERT	1
#define UPDATE	2
#define	DELETE	3
#define FLUSH 	4
#define LIST 	5

/* define the protocol numbers */
#define PROTO_ALL		-1
#define PROTO_ICMP      1
#define PROTO_IGMP  2
#define PROTO_TCP       6
#define PROTO_UDP       17


/* define the structs that are specified in the Design */

/* holds a part that is being held (taken out of order) */
struct stored_packet{
	struct sk_buff *delayed_skb;
	struct nf_info *delayed_info;
	struct stored_packet *next;
};

/* controls reordering going on (does not define reordering rules) */
struct reorder {
	int chosen_burst_size; /* no of packets to delay */
	int delayed_packets; /* no of packets delayed so far, in this burst */
	int chosen_delay_size; /* no of packets to delay by */
	int current_delay; /* how many delayed already */
	struct stored_packet *reorder; /* packets taken out of order */
	struct reorder *next; /* any other reordering going on */
};

struct rules {
	int rule_type;
	int proto; /* transport layer protocol */
	int min_size; /* min packet size for this rule */
	int max_size; /* max size for this rule */
	int probability; /* chance in one thousand */
	int max_burst_size; /* reorder only! max no of packets in burst */
	int max_delay_size;; /* max no of packets to delay by */
	int burst_skew;
	int delay_skew;
	struct reorder *reorder; /* any reordering going on */
	struct rules *next; /* next rule */
};
